@@ -171,8 +171,9 @@ class NicotineFrame:
 
         self.application.add_window(self.MainWindow)
 
-        # Handle Ctrl+C exit gracefully
-        signal.signal(signal.SIGINT, self.on_quit)
+        # Handle Ctrl+C and "kill" exit gracefully
+        for signal_type in (signal.SIGINT, signal.SIGTERM):
+            signal.signal(signal_type, self.on_quit)
 
         self.MainWindow.resize(
             config["ui"]["width"],
@@ -451,6 +452,14 @@ class NicotineFrame:
         except (ImportError, ValueError):
             self.spell_checker = False
 
+    def load_pixbuf_from_path(self, path):
+
+        with open(path, 'rb') as f:
+            loader = GdkPixbuf.PixbufLoader()
+            loader.write(f.read())
+            loader.close()
+            return loader.get_pixbuf()
+
     def get_flag_image(self, country):
 
         if not country:
@@ -460,7 +469,7 @@ class NicotineFrame:
 
         try:
             if country not in self.flag_images:
-                self.flag_images[country] = GdkPixbuf.Pixbuf.new_from_file(
+                self.flag_images[country] = self.load_pixbuf_from_path(
                     os.path.join(self.gui_dir, "icons", "flags", country + ".svg")
                 )
 
@@ -473,7 +482,7 @@ class NicotineFrame:
         """ Load icon required by the UI """
 
         try:
-            return GdkPixbuf.Pixbuf.new_from_file(
+            return self.load_pixbuf_from_path(
                 os.path.join(self.gui_dir, "icons", name + ".svg")
             )
 
@@ -497,7 +506,7 @@ class NicotineFrame:
 
                     if os.path.isfile(path):
                         try:
-                            self.images[name] = GdkPixbuf.Pixbuf.new_from_file(path)
+                            self.images[name] = self.load_pixbuf_from_path(path)
                             loaded = True
 
                         except Exception as e:
@@ -543,7 +552,7 @@ class NicotineFrame:
                         pass
 
                     for name in ("n", "notify"):
-                        self.images[name] = GdkPixbuf.Pixbuf.new_from_file(entry.path)
+                        self.images[name] = self.load_pixbuf_from_path(entry.path)
 
         except FileNotFoundError:
             pass
@@ -566,7 +575,7 @@ class NicotineFrame:
                             # Python 3.5 compatibility
                             pass
 
-                        self.images["trayicon_" + name] = GdkPixbuf.Pixbuf.new_from_file(entry.path)
+                        self.images["trayicon_" + name] = self.load_pixbuf_from_path(entry.path)
 
             except FileNotFoundError:
                 pass
