@@ -31,6 +31,7 @@ from pynicotine.gtkgui.utils import initialise_columns
 from pynicotine.gtkgui.utils import load_ui_elements
 from pynicotine.gtkgui.utils import PopupMenu
 from pynicotine.gtkgui.utils import set_treeview_selected_row
+from pynicotine.gtkgui.utils import show_user_status_tooltip
 from pynicotine.gtkgui.utils import triggers_context_menu
 from pynicotine.gtkgui.utils import update_widget_visuals
 
@@ -145,18 +146,18 @@ class Interests:
         cols = initialise_columns(
             None,
             self.RecommendationUsersList,
-            ["country", _("Country"), 25, "pixbuf", None],
+            ["status", _("Status"), 25, "pixbuf", None],
             ["user", _("User"), 100, "text", None],
             ["speed", _("Speed"), 100, "text", None],
             ["files", _("Files"), 100, "text", None],
         )
 
-        cols["country"].set_sort_column_id(4)
+        cols["status"].set_sort_column_id(4)
         cols["user"].set_sort_column_id(1)
         cols["speed"].set_sort_column_id(5)
         cols["files"].set_sort_column_id(6)
 
-        cols["country"].get_widget().hide()
+        cols["status"].get_widget().hide()
 
         self.recommendation_users = {}
         self.recommendation_users_model = Gtk.ListStore(
@@ -164,7 +165,7 @@ class Interests:
             str,                  # (1) user
             str,                  # (2) hspeed
             str,                  # (3) hfiles
-            GObject.TYPE_INT64,   # (4) status
+            int,                  # (4) status
             GObject.TYPE_UINT64,  # (5) speed
             GObject.TYPE_UINT64   # (6) file count
         )
@@ -184,6 +185,9 @@ class Interests:
 
         self.update_visuals()
 
+    def on_tooltip(self, widget, x, y, keyboard_mode, tooltip):
+        return show_user_status_tooltip(widget, x, y, tooltip, 4)
+
     def on_add_thing_i_like(self, widget, *args):
         thing = widget.get_text()
         widget.set_text("")
@@ -193,7 +197,7 @@ class Interests:
             self.np.config.sections["interests"]["likes"].append(thing)
             self.likes[thing] = self.likes_model.append([thing])
             self.np.config.write_configuration()
-            self.np.queue.put(slskmessages.AddThingILike(thing))
+            self.np.queue.append(slskmessages.AddThingILike(thing))
 
     def on_add_thing_i_dislike(self, widget, *args):
         thing = widget.get_text()
@@ -204,7 +208,7 @@ class Interests:
             self.np.config.sections["interests"]["dislikes"].append(thing)
             self.dislikes[thing] = self.dislikes_model.append([thing])
             self.np.config.write_configuration()
-            self.np.queue.put(slskmessages.AddThingIHate(thing))
+            self.np.queue.append(slskmessages.AddThingIHate(thing))
 
     def on_remove_thing_i_like(self, widget):
         thing = self.til_popup_menu.get_user()
@@ -217,7 +221,7 @@ class Interests:
         self.np.config.sections["interests"]["likes"].remove(thing)
 
         self.np.config.write_configuration()
-        self.np.queue.put(slskmessages.RemoveThingILike(thing))
+        self.np.queue.append(slskmessages.RemoveThingILike(thing))
 
     def on_remove_thing_i_dislike(self, widget):
         thing = self.tidl_popup_menu.get_user()
@@ -230,7 +234,7 @@ class Interests:
         self.np.config.sections["interests"]["dislikes"].remove(thing)
 
         self.np.config.write_configuration()
-        self.np.queue.put(slskmessages.RemoveThingIHate(thing))
+        self.np.queue.append(slskmessages.RemoveThingIHate(thing))
 
     def on_like_recommendation(self, widget):
         thing = widget.get_parent().get_user()
@@ -241,7 +245,7 @@ class Interests:
             self.likes[thing] = self.likes_model.append([thing])
 
             self.np.config.write_configuration()
-            self.np.queue.put(slskmessages.AddThingILike(thing))
+            self.np.queue.append(slskmessages.AddThingILike(thing))
 
         elif not widget.get_active() and \
                 thing and thing in self.np.config.sections["interests"]["likes"]:
@@ -250,7 +254,7 @@ class Interests:
             self.np.config.sections["interests"]["likes"].remove(thing)
 
             self.np.config.write_configuration()
-            self.np.queue.put(slskmessages.RemoveThingILike(thing))
+            self.np.queue.append(slskmessages.RemoveThingILike(thing))
 
     def on_dislike_recommendation(self, widget):
         thing = widget.get_parent().get_user()
@@ -261,7 +265,7 @@ class Interests:
             self.dislikes[thing] = self.dislikes_model.append([thing])
 
             self.np.config.write_configuration()
-            self.np.queue.put(slskmessages.AddThingIHate(thing))
+            self.np.queue.append(slskmessages.AddThingIHate(thing))
 
         elif not widget.get_active() and \
                 thing and thing in self.np.config.sections["interests"]["dislikes"]:
@@ -270,17 +274,17 @@ class Interests:
             self.np.config.sections["interests"]["dislikes"].remove(thing)
 
             self.np.config.write_configuration()
-            self.np.queue.put(slskmessages.RemoveThingIHate(thing))
+            self.np.queue.append(slskmessages.RemoveThingIHate(thing))
 
     def on_recommend_item(self, widget):
         thing = self.til_popup_menu.get_user()
-        self.np.queue.put(slskmessages.ItemRecommendations(thing))
-        self.np.queue.put(slskmessages.ItemSimilarUsers(thing))
+        self.np.queue.append(slskmessages.ItemRecommendations(thing))
+        self.np.queue.append(slskmessages.ItemSimilarUsers(thing))
 
     def on_recommend_recommendation(self, widget):
         thing = self.r_popup_menu.get_user()
-        self.np.queue.put(slskmessages.ItemRecommendations(thing))
-        self.np.queue.put(slskmessages.ItemSimilarUsers(thing))
+        self.np.queue.append(slskmessages.ItemRecommendations(thing))
+        self.np.queue.append(slskmessages.ItemSimilarUsers(thing))
 
     def on_recommend_search(self, widget):
         thing = widget.get_parent().get_user()
@@ -288,13 +292,13 @@ class Interests:
         self.frame.change_main_page("search")
 
     def on_global_recommendations_clicked(self, widget):
-        self.np.queue.put(slskmessages.GlobalRecommendations())
+        self.np.queue.append(slskmessages.GlobalRecommendations())
 
     def on_recommendations_clicked(self, widget):
-        self.np.queue.put(slskmessages.Recommendations())
+        self.np.queue.append(slskmessages.Recommendations())
 
     def on_similar_users_clicked(self, widget):
-        self.np.queue.put(slskmessages.SimilarUsers())
+        self.np.queue.append(slskmessages.SimilarUsers())
 
     def set_recommendations(self, title, recom):
         self.recommendations_model.clear()
@@ -331,7 +335,9 @@ class Interests:
         for user in msg.users:
             iterator = self.recommendation_users_model.append([self.frame.images["offline"], user, "0", "0", 0, 0, 0])
             self.recommendation_users[user] = iterator
-            self.np.queue.put(slskmessages.AddUser(user))
+
+            # Request user status, speed and number of shared files
+            self.np.watch_user(user, force_update=True)
 
     def get_user_status(self, msg):
         if msg.user not in self.recommendation_users:
