@@ -32,30 +32,34 @@ from gi.repository import Gio
 from gi.repository import GLib
 from gi.repository import GObject
 from gi.repository import Gtk
-from gi.repository import Pango
 
 from pynicotine import slskmessages
 from pynicotine.gtkgui.chatrooms import get_completion
 from pynicotine.gtkgui.dialogs import option_dialog
 from pynicotine.gtkgui.utils import add_alias
 from pynicotine.gtkgui.utils import append_line
+from pynicotine.gtkgui.utils import auto_replace
+from pynicotine.gtkgui.utils import censor_chat
+from pynicotine.gtkgui.utils import delete_log
+from pynicotine.gtkgui.utils import entry_completion_find_match
+from pynicotine.gtkgui.utils import entry_completion_found_match
 from pynicotine.gtkgui.utils import expand_alias
+from pynicotine.gtkgui.utils import get_user_status_color
 from pynicotine.gtkgui.utils import IconNotebook
 from pynicotine.gtkgui.utils import is_alias
+from pynicotine.gtkgui.utils import keyval_to_hardware_keycode
 from pynicotine.gtkgui.utils import load_ui_elements
 from pynicotine.gtkgui.utils import open_log
 from pynicotine.gtkgui.utils import PopupMenu
 from pynicotine.gtkgui.utils import scroll_bottom
 from pynicotine.gtkgui.utils import TextSearchBar
-from pynicotine.gtkgui.utils import set_widget_color
-from pynicotine.gtkgui.utils import set_widget_font
 from pynicotine.gtkgui.utils import triggers_context_menu
 from pynicotine.gtkgui.utils import unalias
+from pynicotine.gtkgui.utils import update_tag_visuals
 from pynicotine.gtkgui.utils import update_widget_visuals
 from pynicotine.logfacility import log
-from pynicotine.utils import clean_file
+from pynicotine.utils import get_path
 from pynicotine.utils import version
-from pynicotine.utils import write_log
 
 
 CTCP_VERSION = "\x01VERSION\x01"
@@ -90,9 +94,10 @@ class PrivateChats(IconNotebook):
 
         self.popup_enable()
 
-        self.connected = 1
+        self.connected = True
         self.users = {}
         self.clist = []
+        self.private_message_queue = {}
 
         self.notebook.connect("switch-page", self.on_switch_chat)
 
@@ -125,16 +130,6 @@ class PrivateChats(IconNotebook):
                 if user in self.frame.hilites["private"]:
                     self.frame.notifications.clear("private", tab.user)
 
-    def focused(self, page, focused):
-
-        if not focused:
-            return
-
-        for user, tab in list(self.users.items()):
-            if tab.Main == page:
-                if user in self.frame.hilites["private"]:
-                    self.frame.notifications.clear("private", tab.user)
-
     def get_user_status(self, msg):
 
         if msg.user in self.users:
@@ -146,20 +141,20 @@ class PrivateChats(IconNotebook):
     def send_message(self, user, text=None, show_user=False, bytestring=False):
 
         if user not in self.users:
-            tab = PrivateChat(self, user)
-            self.users[user] = tab
-
-            if user not in self.frame.np.config.sections["privatechat"]["users"]:
-                self.frame.np.config.sections["privatechat"]["users"].append(user)
-
             try:
                 status = self.frame.np.users[user].status
             except Exception:
                 # Offline
                 status = 0
 
-            if user not in self.frame.np.watchedusers:
-                self.frame.np.queue.put(slskmessages.AddUser(user))
+            tab = PrivateChat(self, user, status)
+            self.users[user] = tab
+
+            if user not in self.frame.np.config.sections["privatechat"]["users"]:
+                self.frame.np.config.sections["privatechat"]["users"].append(user)
+
+            # Get notified of user status
+            self.frame.np.watch_user(user)
 
             self.append_page(tab.Main, user, tab.on_close, status=status)
 
@@ -182,46 +177,72 @@ class PrivateChats(IconNotebook):
         menu.get_items()[_("Send _Message")].set_visible(False)
 
         menu.append_item(("", None))
-        menu.append_item(("#" + _("Close All Tabs"), menu.on_close_all_tabs, self))
+        menu.append_item(("#" + _("Close All Tabs"), self.users[username].on_close_all_tabs))
         menu.append_item(("#" + _("_Close Tab"), self.users[username].on_close))
         menu.toggle_user_items()
 
         menu.popup()
         return True
 
-    def on_tab_click(self, widget, event, page):
+    def private_message_queue_add(self, msg, text):
 
-        if triggers_context_menu(event):
-            return self.on_tab_popup(widget, page)
+        user = msg.user
 
-        if event.button == 2:
-            username = self.get_page_owner(page, self.users)
-            self.users[username].on_close(widget)
-            return True
+        if user not in self.private_message_queue:
+            self.private_message_queue[user] = [[msg, text]]
+        else:
+            self.private_message_queue[user].append([msg, text])
 
-        return False
+    def private_message_queue_process(self, user):
 
-    def close_all_tabs(self, dialog, response, data):
+        if user not in self.private_message_queue:
+            return
 
-        if response == Gtk.ResponseType.OK:
-            for user in self.users.copy():
-                self.users[user].on_close(dialog)
+        for data in self.private_message_queue[user][:]:
+            msg, text = data
+            self.private_message_queue[user].remove(data)
+            self.show_message(msg, text)
 
-        dialog.destroy()
+    def show_notification(self, user, text):
+
+        chat = self.users[user]
+
+        # Hilight top-level tab label
+        self.frame.request_tab_icon(self.frame.PrivateChatTabLabel)
+
+        # Highlight sub-level tab label
+        self.request_changed(chat.Main)
+
+        # Don't show notifications if the private chat is open and the window
+        # is in use
+        if self.get_current_page() == self.page_num(chat.Main) and \
+           self.frame.MainNotebook.get_current_page() == self.frame.MainNotebook.page_num(self.frame.privatechatvbox) and \
+           self.frame.MainWindow.is_active():
+            return
+
+        # Update tray icon and show urgency hint
+        self.frame.notifications.add("private", user)
+
+        if self.frame.np.config.sections["notifications"]["notification_popup_private_message"]:
+            self.frame.notifications.new_notification(
+                text,
+                title=_("Private message from %s") % user,
+                priority=Gio.NotificationPriority.HIGH
+            )
 
     def show_message(self, msg, text, newmessage=True):
 
-        if msg.user in self.frame.np.config.sections["server"]["ignorelist"]:
+        if self.frame.np.network_filter.is_user_ignored(msg.user):
             return
 
         if msg.user in self.frame.np.users and isinstance(self.frame.np.users[msg.user].addr, tuple):
             ip, port = self.frame.np.users[msg.user].addr
-            if self.frame.np.ip_ignored(ip):
+            if self.frame.np.network_filter.is_ip_ignored(ip):
                 return
 
         elif newmessage:
-            self.frame.np.queue.put(slskmessages.GetPeerAddress(msg.user))
-            self.frame.np.private_message_queue_add(msg, text)
+            self.frame.np.queue.append(slskmessages.GetPeerAddress(msg.user))
+            self.private_message_queue_add(msg, text)
             return
 
         user_text = self.frame.np.pluginhandler.incoming_private_chat_event(msg.user, text)
@@ -231,25 +252,7 @@ class PrivateChats(IconNotebook):
         (u, text) = user_text
 
         self.send_message(msg.user, None)
-        chat = self.users[msg.user]
-        self.request_changed(chat.Main)
-
-        # Hilight main private chats Label
-        self.frame.request_tab_icon(self.frame.PrivateChatTabLabel)
-
-        # Show notifications if the private chats notebook isn't selected,
-        # the tab is not selected, or the main window isn't mapped
-        if self.get_current_page() != self.page_num(chat.Main) or \
-           self.frame.MainNotebook.get_current_page() != self.frame.MainNotebook.page_num(self.frame.privatechatvbox) or \
-           not self.frame.MainWindow.get_property("visible"):
-            self.frame.notifications.add("private", msg.user)
-
-            if self.frame.np.config.sections["notifications"]["notification_popup_private_message"]:
-                self.frame.notifications.new_notification(
-                    text,
-                    title=_("Private message from %s") % msg.user,
-                    priority=Gio.NotificationPriority.HIGH
-                )
+        self.show_notification(msg.user, text)
 
         # SEND CLIENT VERSION to user if the following string is sent
         ctcpversion = 0
@@ -286,10 +289,14 @@ class PrivateChats(IconNotebook):
 
     def login(self):
 
-        self.connected = 1
+        self.connected = True
+
         for user in self.users:
             self.users[user].login()
 
+            # Get notified of user status
+            self.frame.np.watch_user(user)
+
         if not self.frame.np.config.sections["privatechat"]["store"]:
             return
 
@@ -299,7 +306,7 @@ class PrivateChats(IconNotebook):
 
     def conn_close(self):
 
-        self.connected = 0
+        self.connected = False
 
         for user in self.users:
             self.users[user].conn_close()
@@ -329,7 +336,7 @@ class PrivateChats(IconNotebook):
 
 class PrivateChat:
 
-    def __init__(self, chats, user):
+    def __init__(self, chats, user, status):
 
         self.user = user
         self.chats = chats
@@ -342,7 +349,7 @@ class PrivateChat:
 
         self.autoreplied = 0
         self.offlinemessage = 0
-        self.status = -1
+        self.status = status
         self.clist = []
 
         # Text Search
@@ -365,8 +372,8 @@ class PrivateChat:
         completion.set_model(liststore)
 
         completion.set_text_column(0)
-        completion.set_match_func(self.frame.entry_completion_find_match, self.ChatLine)
-        completion.connect("match-selected", self.frame.entry_completion_found_match, self.ChatLine)
+        completion.set_match_func(entry_completion_find_match)
+        completion.connect("match-selected", entry_completion_found_match)
 
         self.Log.set_active(self.frame.np.config.sections["logging"]["privatechat"])
 
@@ -403,7 +410,7 @@ class PrivateChat:
 
         # Read log file
         config = self.frame.np.config.sections
-        log = os.path.join(config["logging"]["privatelogsdir"], clean_file(self.user.replace(os.sep, "-")) + ".log")
+        filename = self.user.replace(os.sep, "-") + ".log"
 
         try:
             numlines = int(config["logging"]["readprivatelines"])
@@ -411,20 +418,24 @@ class PrivateChat:
             numlines = 15
 
         try:
-            try:
-                self.append_log_lines(log, numlines, 'utf-8')
-
-            except UnicodeDecodeError:
-                self.append_log_lines(log, numlines, 'latin-1')
+            get_path(config["logging"]["privatelogsdir"], filename, self.append_log_lines, numlines)
 
         except IOError:
             pass
 
         GLib.idle_add(scroll_bottom, self.ChatScroll.get_parent())
 
-    def append_log_lines(self, log, numlines, encoding='utf-8'):
+    def append_log_lines(self, path, numlines):
 
-        with open(log, 'r', encoding=encoding) as lines:
+        try:
+            self._append_log_lines(path, numlines, 'utf-8')
+
+        except UnicodeDecodeError:
+            self._append_log_lines(path, numlines, 'latin-1')
+
+    def _append_log_lines(self, path, numlines, encoding='utf-8'):
+
+        with open(path, 'r', encoding=encoding) as lines:
             # Only show as many log lines as specified in config
             lines = deque(lines, numlines)
 
@@ -480,21 +491,7 @@ class PrivateChat:
     def delete_chat_log_response(self, dialog, response, data):
 
         if response == Gtk.ResponseType.OK:
-            log_path = os.path.join(
-                self.frame.np.config.sections["logging"]["privatelogsdir"],
-                clean_file(self.user.replace(os.sep, "-")) + ".log"
-            )
-
-            try:
-                if os.path.exists(log_path):
-                    os.remove(log_path)
-
-            except Exception as e:
-                log.add(_("Failed to remove logged chat messages for user '%(user)s'. Error: %(error)s"), {
-                    "user": self.user,
-                    "error": e
-                })
-
+            delete_log(self.frame.np.config.sections["logging"]["privatelogsdir"], self.user)
             self.on_clear_messages(dialog)
 
         dialog.destroy()
@@ -527,15 +524,13 @@ class PrivateChat:
 
     def show_message(self, text, newmessage=True, timestamp=None):
 
-        self.create_tags()
-
         if text[:4] == "/me ":
-            line = "* %s %s" % (self.user, self.frame.censor_chat(text[4:]))
+            line = "* %s %s" % (self.user, censor_chat(text[4:]))
             speech = line[2:]
             tag = self.tag_me
         else:
-            line = "[%s] %s" % (self.user, self.frame.censor_chat(text))
-            speech = self.frame.censor_chat(text)
+            line = "[%s] %s" % (self.user, censor_chat(text))
+            speech = censor_chat(text)
             tag = self.tag_remote
 
         timestamp_format = self.frame.np.config.sections["logging"]["private_timestamp"]
@@ -566,7 +561,7 @@ class PrivateChat:
 
         if self.Log.get_active():
             timestamp_format = self.frame.np.config.sections["logging"]["log_timestamp"]
-            write_log(self.frame.np.config.sections["logging"]["privatelogsdir"], self.user, line, timestamp_format)
+            log.write_log(self.frame.np.config.sections["logging"]["privatelogsdir"], self.user, line, timestamp_format)
 
         autoreply = self.frame.np.config.sections["server"]["autoreply"]
         if self.frame.away and not self.autoreplied and autoreply:
@@ -609,33 +604,21 @@ class PrivateChat:
 
         if self.Log.get_active():
             timestamp_format = self.frame.np.config.sections["logging"]["log_timestamp"]
-            write_log(self.frame.np.config.sections["logging"]["privatelogsdir"], self.user, line, timestamp_format)
+            log.write_log(self.frame.np.config.sections["logging"]["privatelogsdir"], self.user, line, timestamp_format)
 
         if bytestring:
             payload = text
         else:
-            payload = self.frame.auto_replace(text)
+            payload = auto_replace(text)
 
         if self.PeerPrivateMessages.get_active():
             # not in the soulseek protocol
             self.frame.np.send_message_to_peer(self.user, slskmessages.PMessageUser(None, my_username, payload))
         else:
-            self.frame.np.queue.put(slskmessages.MessageUser(self.user, payload))
+            self.frame.np.queue.append(slskmessages.MessageUser(self.user, payload))
 
         self.frame.np.pluginhandler.outgoing_private_chat_notification(self.user, text)
 
-    def thread_alias(self, alias):
-
-        text = expand_alias(alias)
-        if not text:
-            log.add(_('Alias "%s" returned nothing'), alias)
-            return
-
-        if text[:2] == "//":
-            text = text[1:]
-
-        self.frame.np.queue.put(slskmessages.MessageUser(self.user, self.frame.auto_replace(text)))
-
     def on_enter(self, widget):
 
         text = widget.get_text()
@@ -645,8 +628,16 @@ class PrivateChat:
             return
 
         if is_alias(text):
-            import _thread
-            _thread.start_new_thread(self.thread_alias, (text,))
+            new_text = expand_alias(text)
+
+            if not new_text:
+                log.add(_('Alias "%s" returned nothing'), text)
+                return
+
+            if new_text[:2] == "//":
+                new_text = new_text[1:]
+
+            self.frame.np.queue.append(slskmessages.MessageUser(self.user, auto_replace(new_text)))
             widget.set_text("")
             return
 
@@ -674,7 +665,7 @@ class PrivateChat:
 
         elif cmd in ("/join", "/j"):
             if args:
-                self.frame.np.queue.put(slskmessages.JoinRoom(args))
+                self.frame.np.queue.append(slskmessages.JoinRoom(args))
 
         elif cmd in ("/w", "/whois", "/info"):
             self.frame.local_user_info_request(arg_self)
@@ -686,7 +677,7 @@ class PrivateChat:
 
         elif cmd == "/ip":
             self.frame.np.ip_requested.add(arg_self)
-            self.frame.np.queue.put(slskmessages.GetPeerAddress(arg_self))
+            self.frame.np.queue.append(slskmessages.GetPeerAddress(arg_self))
 
         elif cmd == "/pm":
             if args:
@@ -739,23 +730,23 @@ class PrivateChat:
 
         elif cmd == "/ban":
             if args:
-                self.frame.ban_user(args)
+                self.frame.np.network_filter.ban_user(args)
 
         elif cmd == "/ignore":
             if args:
-                self.frame.ignore_user(args)
+                self.frame.np.network_filter.ignore_user(args)
 
         elif cmd == "/ignoreip":
             if args:
-                self.frame.ignore_ip(args)
+                self.frame.np.network_filter.ignore_ip(args)
 
         elif cmd == "/unban":
             if args:
-                self.frame.unban_user(args)
+                self.frame.np.network_filter.unban_user(args)
 
         elif cmd == "/unignore":
             if args:
-                self.frame.unignore_user(args)
+                self.frame.np.network_filter.unignore_user(args)
 
         elif cmd == "/ctcpversion":
             if arg_self:
@@ -816,8 +807,7 @@ class PrivateChat:
     def create_tag(self, buffer, color):
 
         tag = buffer.create_tag()
-        set_widget_color(tag, self.frame.np.config.sections["ui"][color])
-        set_widget_font(tag, self.frame.np.config.sections["ui"]["chatfont"])
+        update_tag_visuals(tag, color)
 
         return tag
 
@@ -829,14 +819,8 @@ class PrivateChat:
         self.tag_me = self.create_tag(buffer, "chatme")
         self.tag_hilite = self.create_tag(buffer, "chathilite")
 
-        if self.status == 1 and self.frame.np.config.sections["ui"]["showaway"]:
-            statuscolor = "useraway"
-        elif self.status == 2 or not self.frame.np.config.sections["ui"]["showaway"] and self.status == 1:
-            statuscolor = "useronline"
-        else:
-            statuscolor = "useroffline"
-
-        self.tag_username = self.create_tag(buffer, statuscolor)
+        color = get_user_status_color(self.status)
+        self.tag_username = self.create_tag(buffer, color)
 
         if self.chats.connected:
             if self.frame.away and self.frame.np.config.sections["ui"]["showaway"]:
@@ -846,84 +830,23 @@ class PrivateChat:
         else:
             self.tag_my_username = self.create_tag(buffer, "useroffline")
 
-        usernamestyle = self.frame.np.config.sections["ui"]["usernamestyle"]
-
-        if usernamestyle == "bold":
-            self.tag_username.set_property("weight", Pango.Weight.BOLD)
-            self.tag_my_username.set_property("weight", Pango.Weight.BOLD)
-        else:
-            self.tag_username.set_property("weight", Pango.Weight.NORMAL)
-            self.tag_my_username.set_property("weight", Pango.Weight.NORMAL)
-
-        if usernamestyle == "italic":
-            self.tag_username.set_property("style", Pango.Style.ITALIC)
-            self.tag_my_username.set_property("style", Pango.Style.ITALIC)
-        else:
-            self.tag_username.set_property("style", Pango.Style.NORMAL)
-            self.tag_my_username.set_property("style", Pango.Style.NORMAL)
-
-        if usernamestyle == "underline":
-            self.tag_username.set_property("underline", Pango.Underline.SINGLE)
-            self.tag_my_username.set_property("underline", Pango.Underline.SINGLE)
-        else:
-            self.tag_username.set_property("underline", Pango.Underline.NONE)
-            self.tag_my_username.set_property("underline", Pango.Underline.NONE)
-
-    def update_tag_visuals(self, tag, color):
-
-        set_widget_color(tag, self.frame.np.config.sections["ui"][color])
-        set_widget_font(tag, self.frame.np.config.sections["ui"]["chatfont"])
-
-        if color in ("useraway", "useronline", "useroffline"):
-
-            usernamestyle = self.frame.np.config.sections["ui"]["usernamestyle"]
-
-            if usernamestyle == "bold":
-                tag.set_property("weight", Pango.Weight.BOLD)
-            else:
-                tag.set_property("weight", Pango.Weight.NORMAL)
-
-            if usernamestyle == "italic":
-                tag.set_property("style", Pango.Style.ITALIC)
-            else:
-                tag.set_property("style", Pango.Style.NORMAL)
-
-            if usernamestyle == "underline":
-                tag.set_property("underline", Pango.Underline.SINGLE)
-            else:
-                tag.set_property("underline", Pango.Underline.NONE)
-
     def update_tags(self):
 
-        self.update_tag_visuals(self.tag_remote, "chatremote")
-        self.update_tag_visuals(self.tag_local, "chatlocal")
-        self.update_tag_visuals(self.tag_me, "chatme")
-        self.update_tag_visuals(self.tag_hilite, "chathilite")
+        update_tag_visuals(self.tag_remote, "chatremote")
+        update_tag_visuals(self.tag_local, "chatlocal")
+        update_tag_visuals(self.tag_me, "chatme")
+        update_tag_visuals(self.tag_hilite, "chathilite")
 
-        color = self.get_user_status_color(self.status)
-        self.update_tag_visuals(self.tag_username, color)
+        color = get_user_status_color(self.status)
+        update_tag_visuals(self.tag_username, color)
 
         if self.chats.connected:
             if self.frame.away and self.frame.np.config.sections["ui"]["showaway"]:
-                self.update_tag_visuals(self.tag_my_username, "useraway")
+                update_tag_visuals(self.tag_my_username, "useraway")
             else:
-                self.update_tag_visuals(self.tag_my_username, "useronline")
+                update_tag_visuals(self.tag_my_username, "useronline")
         else:
-            self.update_tag_visuals(self.tag_my_username, "useroffline")
-
-    def get_user_status_color(self, status):
-
-        if status == 1:
-            color = "useraway"
-        elif status == 2:
-            color = "useronline"
-        else:
-            color = "useroffline"
-
-        if not self.frame.np.config.sections["ui"]["showaway"] and color == "useraway":
-            color = "useronline"
-
-        return color
+            update_tag_visuals(self.tag_my_username, "useroffline")
 
     def get_user_status(self, status):
 
@@ -931,12 +854,9 @@ class PrivateChat:
             return
 
         self.status = status
-        color = self.get_user_status_color(self.status)
-        self.update_tag_visuals(self.tag_username, color)
 
-    def on_close(self, widget):
-
-        self.chats.remove_tab(self)
+        color = get_user_status_color(self.status)
+        update_tag_visuals(self.tag_username, color)
 
     def get_completion_list(self, ix=0, text="", clist=None):
 
@@ -977,25 +897,9 @@ class PrivateChat:
 
     def on_key_press(self, widget, event):
 
-        if event.keyval == Gdk.keyval_from_name("Prior"):
+        keycode = event.hardware_keycode
 
-            scrolled = self.ChatScroll.get_parent()
-            adj = scrolled.get_vadjustment()
-            adj.set_value(adj.value - adj.page_increment)
-
-        elif event.keyval == Gdk.keyval_from_name("Next"):
-
-            scrolled = self.ChatScroll.get_parent()
-            adj = scrolled.get_vadjustment()
-            maximum = adj.upper - adj.page_size
-            new = adj.value + adj.page_increment
-
-            if new > maximum:
-                new = maximum
-
-            adj.set_value(new)
-
-        if event.keyval != Gdk.keyval_from_name("Tab"):
+        if keycode not in keyval_to_hardware_keycode(Gdk.KEY_Tab):
             return False
 
         config = self.frame.np.config.sections["words"]
@@ -1014,5 +918,10 @@ class PrivateChat:
             widget.set_position(preix + len(completion))
 
         widget.stop_emission_by_name("key_press_event")
-
         return True
+
+    def on_close(self, widget):
+        self.chats.remove_tab(self)
+
+    def on_close_all_tabs(self, widget):
+        self.chats.remove_all_pages()
