@@ -113,7 +113,7 @@ class Tray:
 
         users.sort()
         user = combo_box_dialog(
-            parent=self.frame.MainWindow,
+            parent=self.frame.application.get_active_window(),
             title=GLib.get_application_name() + ": " + _("Start Messaging"),
             message=_('Enter the User who you wish to send a private message:'),
             droplist=users
@@ -134,7 +134,7 @@ class Tray:
         users.sort()
 
         user = combo_box_dialog(
-            parent=self.frame.MainWindow,
+            parent=self.frame.application.get_active_window(),
             title=GLib.get_application_name() + ": " + _("Get User Info"),
             message=_('Enter the User whose User Info you wish to receive:'),
             droplist=users
@@ -152,7 +152,7 @@ class Tray:
         users.sort()
 
         user = combo_box_dialog(
-            parent=self.frame.MainWindow,
+            parent=self.frame.application.get_active_window(),
             title=GLib.get_application_name() + ": " + _("Get A User's IP"),
             message=_('Enter the User whose IP Address you wish to receive:'),
             droplist=users
@@ -160,7 +160,7 @@ class Tray:
 
         if user:
             self.frame.np.ip_requested.add(user)
-            self.frame.np.queue.put(slskmessages.GetPeerAddress(user))
+            self.frame.np.queue.append(slskmessages.GetPeerAddress(user))
 
     def on_get_a_users_shares(self, widget, prefix=""):
 
@@ -171,7 +171,7 @@ class Tray:
         users.sort()
 
         user = combo_box_dialog(
-            parent=self.frame.MainWindow,
+            parent=self.frame.application.get_active_window(),
             title=GLib.get_application_name() + ": " + _("Get A User's Shares List"),
             message=_('Enter the User whose Shares List you wish to receive:'),
             droplist=users
@@ -318,6 +318,7 @@ class Tray:
         return True
 
     def set_image(self, status=None):
+
         if not self.is_tray_icon_visible():
             return
 
@@ -326,11 +327,11 @@ class Tray:
                 self.tray_status["status"] = status
 
             # Check for hilites, and display hilite icon if there is a room or private hilite
-            if self.frame.hilites["rooms"] == [] and self.frame.hilites["private"] == []:
+            if self.frame.hilites["rooms"] or self.frame.hilites["private"]:
+                icon_name = "msg"
+            else:
                 # If there is no hilite, display the status
                 icon_name = self.tray_status["status"]
-            else:
-                icon_name = "msg"
 
             if icon_name != self.tray_status["last"]:
                 self.tray_status["last"] = icon_name
