@@ -693,7 +693,9 @@ class MessageAcked(ServerMessage):
 class FileSearch(ServerMessage):
     """ Server code: 26 """
     """ We send this to the server when we search for something. Alternatively,
-    the server sends this message to tell us that someone is searching for something.
+    the server sends this message outside the distributed network to tell us that
+    someone is searching for something, currently used for UserSearch and RoomSearch
+    requests.
 
     The search id is a random number generated by the client and is used to track the
     search results.
@@ -863,7 +865,7 @@ class UserSearch(ServerMessage):
 
         return msg
 
-    # Soulfind support, not used by the official server
+    # Soulfind support, the official server sends a FileSearch message (code 26) instead
     def parse_network_message(self, message):
         pos, self.user = self.get_object(message, str)
         pos, self.searchid = self.get_object(message, int, pos)
@@ -1297,16 +1299,19 @@ class CheckPrivileges(ServerMessage):
         pos, self.seconds = self.get_object(message, int)
 
 
-class SearchRequest(ServerMessage):
+class EmbeddedMessage(ServerMessage):
     """ Server code: 93 """
-    """ The server sends us search requests from other users. """
+    """ The server sends us an embedded distributed message. The only type
+    of distributed message sent at present is DistribSearch (distributed code 3).
+    If we receive such a message, we are a branch root in the distributed network,
+    and we distribute the embedded message (not the unpacked distributed message)
+    to our child peers. """
+
+    __slots__ = "distrib_code", "distrib_message"
 
     def parse_network_message(self, message):
-        pos, self.code = 1, message[0]
-        pos, self.something = self.get_object(message, int, pos)
-        pos, self.user = self.get_object(message, str, pos)
-        pos, self.searchid = self.get_object(message, int, pos)
-        pos, self.searchterm = self.get_object(message, str, pos)
+        self.distrib_code = message[0]
+        self.distrib_message = message[1:]
 
 
 class AcceptChildren(ServerMessage):
@@ -1503,6 +1508,9 @@ class RemoveThingIHate(RemoveThingILike):
 
 class RoomSearch(ServerMessage):
     """ Server code: 120 """
+    """ We send this to the server to search files shared by users who have joined
+    a specific chat room. The ticket/search id is a random number generated by the
+    client and is used to track the search results. """
 
     def __init__(self, room=None, requestid=None, text=""):
         self.room = room
@@ -1517,7 +1525,7 @@ class RoomSearch(ServerMessage):
 
         return msg
 
-    # Soulfind support, not used by the official server
+    # Soulfind support, the official server sends a FileSearch message (code 26) instead
     def parse_network_message(self, message):
         pos, self.user = self.get_object(message, str)
         pos, self.searchid = self.get_object(message, int, pos)
@@ -2622,13 +2630,8 @@ class DistribAlive(DistribMessage):
 
 class DistribSearch(DistribMessage):
     """ Distrib code: 3 """
-    """
-    Search request that arrives through the distributed network.
-    We transmit the search request to our children.
-
-    Search requests are sent to us by the server using SearchRequest
-    if we're a branch root, or by our parent using DistribSearch.
-    """
+    """ Search request that arrives through the distributed network.
+    We transmit the search request to our child peers. """
 
     __slots__ = "unknown", "conn", "user", "searchid", "searchterm"
 
@@ -2680,30 +2683,17 @@ class DistribChildDepth(DistribMessage):
         pos, self.value = self.get_object(message, int)
 
 
-class DistribServerSearch(DistribMessage):
+class DistribEmbeddedMessage(DistribMessage):
     """ Distrib code: 93 """
-    """
-    Search request that arrives through the distributed network.
-    We transmit the search request to our children.
+    """ A branch root sends us an embedded distributed message. The only type
+    of distributed message sent at present is DistribSearch (distributed code 3).
+    We unpack the distributed message and distribute it to our child peers. """
 
-    Search requests are sent to us by the server using SearchRequest
-    if we're a branch root, or by our parent using DistribSearch.
-    """
-
-    __slots__ = "unknown", "conn", "user", "searchid", "searchterm"
+    __slots__ = "conn", "distrib_code", "distrib_message"
 
     def __init__(self, conn):
         self.conn = conn
 
     def parse_network_message(self, message):
-        try:
-            self._parse_network_message(message)
-        except Exception as error:
-            log.add_warning(_("Exception during parsing %(area)s: %(exception)s"), {'area': 'DistribServerSearch', 'exception': error})
-            return False
-
-    def _parse_network_message(self, message):
-        pos, self.unknown = self.get_object(message, int, getunsignedlonglong=True)
-        pos, self.user = self.get_object(message, str, pos)
-        pos, self.searchid = self.get_object(message, int, pos)
-        pos, self.searchterm = self.get_object(message, str, pos)
+        self.distrib_code = message[3]
+        self.distrib_message = message[4:]
